const fs = require("fs");
const path = require("path");
const xlsx = require("xlsx");

const Clients = require("../Model/Clients");
const Organization = require("../Model/Organization");
const Zone = require("../Model/Zone");
const Branch = require("../Model/Branch");
const User = require("../Model/User");
const Empanelment = require("../Model/Empanelment");
const CreditOfficer = require("../Model/CreditOfficer");

module.exports.uploadExcelFile = async (req, res) => {
    try {
        const userId = req.local.user._id; // User details are available in req.local.user
        const organizationData = req.local.user.organization; // Get organization from user details
        const modelName = req.params.modelName; // Model name passed dynamically as a route parameter
        const file = req.file; // Single file uploaded

        // Validate file existence
        if (!file) {
            return res.status(400).send({ status: false, message: "No file found." });
        }

        // Check for valid model
        let Model;
        try {
            Model = require(`../Model/${modelName}`);
        } catch (err) {
            return res.status(400).send({ status: false, message: `Model ${modelName} not found.` });
        }

        // Save file to a specific directory
        const destination = path.join(process.env.CDN_PATH, "uploads", modelName, file.filename);
        fs.mkdirSync(path.dirname(destination), { recursive: true });

        // Move the uploaded file
        if (fs.existsSync(file.path)) {
            fs.renameSync(file.path, destination);
            file.path = destination;
        } else {
            return res.status(500).send({ status: false, message: "File upload failed." });
        }

        // Parse the Excel file
        const workbook = xlsx.readFile(file.path);
        const sheetName = workbook.SheetNames[0]; // Default to the first sheet
        const data = xlsx.utils.sheet_to_json(workbook.Sheets[sheetName]); // Convert sheet to JSON

        const insertedRecords = [];
        for (const row of data) {
            let existingRecord;
            switch (modelName.toLower()) {
                case "clients":
                    existingRecord = await Clients.findOne({ name: row.name });
                    break;
                case "zones":
                    existingRecord = await Zone.findOne({ 'client._id': row.clientId, zoneName: row.zoneName });
                    break;
                case "users":
                    existingRecord = await User.findOne({ email: row.email });s
                    break;
                case "branches":
                    existingRecord = await Branch.findOne({ 'zone._id': row.zoneId, name: row.name });
                    break;
                case "creditofficer":
                    existingRecord = await CreditOfficer.findOne({ 'branch._id': row.branchId });
                    break;
                case "empanelments":
                    existingRecord = await Empanelment.findOne({ empanelmentNo: row.empanelmentNo });
                    break;
                default:
                    break;
            }

            let client = {};
            if (row.clientId) {
                client = await Clients.findOne({ _id: row.clientId });
                if (!client) {
                    return res.status(400).send({ status: false, message: `Client with ID ${row.clientId} not found.` });
                }
            }
            let zone = {};
            if (row.zoneId) {
                zone = await Zone.findOne({ _id: row.zoneId });
                if (!zone) {
                    return res.status(400).send({
                        status: false,
                        message: `Zone with ID ${row.zoneId} not found.`,
                    });
                }
            }

            let branch = {};
            if (row.branchId) {
                branch = await Branch.findOne({ _id: row.branchId });
                if (!branch) {
                    return res.status(400).send({
                        status: false,
                        message: `Branch with ID ${row.branchId} not found.`,
                    });
                }
            }

            let user = {};
            if (row.valuerId) {
                user = await User.findOne({ _id: row.valuerId });
                if (!user) {
                    return res.status(400).send({
                        status: false,
                        message: `User with ID ${row.valuerId} not found.`,
                    });
                }
            }

            const organization = await Organization.findOne({
                _id: organizationData._id,
            });
            if (!existingRecord) {
                if (modelName === "Zone") {
                    const zone = new Model({
                        zoneName: row.zoneName,
                        client,
                        organization,
                        createdBy: userId,
                    });

                    await zone.save();
                    insertedRecords.push(zone);
                } else if (modelName === "Clients") {
                    const address = {
                        street: row.street,
                        city: row.city,
                        state: row.state,
                        postalCode: row.postalCode,
                        country: row.country,
                    };
                    const client = new Model({
                        ...row,
                        address,
                        createdBy: userId,
                        organization,
                    });
                    await client.save();
                    insertedRecords.push(client);
                } else if (modelName === "Branch") {
                    const branch = new Model({
                        ...row,
                        client,
                        organization,
                        zone,
                        createdBy: userId,
                    });
                    await branch.save();
                    insertedRecords.push(branch);
                } else if (modelName === "CreditOfficer") {
                    const creditOfficer = new Model({
                        ...row,
                        client,
                        branch,
                        organization,
                        createdBy: userId,
                    });
                    await creditOfficer.save();
                    insertedRecords.push(creditOfficer);
                } else if (modelName === "User") {
                    // Process Excel serial dates for birthDate
                    if (row.birthDate && !isNaN(row.birthDate)) {
                        const excelSerialDate = row.birthDate;
                        const jsDate = new Date((excelSerialDate - 25569) * 86400 * 1000);

                        const formattedDate = `${jsDate.getMonth() + 1}/${jsDate.getDate()}/${jsDate.getFullYear()}`;
                        row.birthDate = formattedDate;
                    }
                    const user = new Model({
                        ...row,
                        organization,
                        createdBy: userId,
                    });
                    await user.save();
                    insertedRecords.push(user);
                } else if (modelName === "Empanelment") {
                    // Process Excel serial dates for startDate
                    if (row.startDate && !isNaN(row.startDate)) {
                        const excelSerialDate = row.startDate;
                        const jsDate = new Date((excelSerialDate - 25569) * 86400 * 1000);
                        const formattedDate = `${jsDate.getMonth() + 1}/${jsDate.getDate()}/${jsDate.getFullYear()}`;
                        row.startDate = formattedDate;
                    }

                    // Process Excel serial dates for endDate
                    if (row.endDate && !isNaN(row.endDate)) {
                        const excelSerialDate = row.endDate;
                        const jsDate = new Date((excelSerialDate - 25569) * 86400 * 1000);
                        const formattedDate = `${jsDate.getMonth() + 1}/${jsDate.getDate()}/${jsDate.getFullYear()}`;
                        row.endDate = formattedDate;
                    }
                    const empanelment = new Model({
                        ...row,
                        zone,
                        user,
                        organization,
                        createdBy: userId,
                    });
                    await empanelment.save();
                    insertedRecords.push(empanelment);
                }
            }
        }

        return res.status(200).send({
            status: true,
            message: `${insertedRecords.length} new records inserted.`,
            file: file.filename,
        });
    } catch (e) {
        console.error(e);
        return res.status(500).send({ status: false, error: e.message });
    }
};