const CreateContactServices = require('../Services/CreateContact.services');
const OrganizationServices = require('../Services/Organization.services');
const { logger } = require('../logger');

/**
 * @description This function is used for get all createContacts list from the database  
 * @param {*} req 
 * @param {*} res 
 * @returns Object CreateContactResult { status : true, CreateContacts : array }
 * - { status: false, error: string }
 */
module.exports.getAllCreateContacts = async (req, res) => {
    try {
        let query = {};
        // query['organizationId'] = req.local.user.organization._id;
        query['status'] = "TRUE";

        var count = 0;
        const searchValue = req.query.searchValue;

        if (searchValue) {
            query["$or"] = [
                { type: { $regex: searchValue, $options: 'i' } },
                { details: { $regex: searchValue, $options: 'i' } },
                { 'createdBy.firstName': { $regex: searchValue, $options: 'i' } },
                { 'createdBy.lastName': { $regex: searchValue, $options: 'i' } }
            ]
        }

        // For Advanced Search By ID
        var createdBy = req.query.createdBy;
        if (createdBy) query['createdBy._id'] = createdBy

        const CreateContactResultCount = await CreateContactServices.getCount(query)
        if (!CreateContactResultCount.status || CreateContactResultCount.error) return res.status(500).send(CreateContactResultCount);
        count = CreateContactResultCount.CreateContacts;

        const CreateContactResult = await CreateContactServices.getAllCreateContacts(query, req.local.limit, req.local.startIndex);
        if (!CreateContactResult.status || CreateContactResult.error) return res.status(500).send(CreateContactResult);

        return res.status(200).send({ status: true, CreateContacts: CreateContactResult.CreateContacts, count: count });
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.getAllCreateContacts= ~ e:`, e)
        return res.status(500).send({ status: false, error: e.toString() });
    }
};
/**
 * @description This function is used for get only one Createcontact 
 * @param {string} req.params from that get createContact Id  
 * @param {*} res 
 * @returns Object CreateContactResult { status : true, CreateContacts : object }
 * - { status: false, error: string }
 */
module.exports.getCreateContact = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const CreateContactResult = await CreateContactServices.getCreateContact(id);
        if (!CreateContactResult.status || CreateContactResult.error) return res.status(500).send(CreateContactResult);

        return res.status(200).send(CreateContactResult);
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.getCreateContact= ~ e:`, e)
        return res.status(500).send({ status: false, error: e.toString() });
    }
};
/**
 * This function is used for create new createContact 
 * @param {*} req.local.createContact from that get createContact Object 
 * @param {*} res 
 * @returns Object CreateContactResult { status: true, CreateContacts: object }
 * -{ status: false, error: string }
 */

module.exports.createCreateContact = async (req, res) => {
    try {
        const datas = req.local.createContact;
        const user = req.local.user;
        var images = [];
        // console.log(req.files)
        if (req.files) {
            req.files.images.map(async (img) => {
                images.push(img.path)
            })
            datas.image = images
        }

        datas.createdBy = user;
        datas.createDate = new Date();
        datas.organizationId = req.local.user.organization._id;

        const CreateContactResult = await CreateContactServices.createCreateContact(datas);
        if (!CreateContactResult.status || CreateContactResult.error) return res.status(500).send(CreateContactResult);

        return res.status(200).send(CreateContactResult);

    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.createCreateContact= ~ err:`, err)
        return res.status(500).send({ status: false, error: err.toString() });
    }
};

/**
 * @description This function is used for update createContact 
 * @param {*} req.params.id from that get createContacId and from req.local.createContact get Object
 * @param {*} res 
 * @returns Object CreateContact_res { status: true, CreateContacts: object }
 * - { status : false, error : string }
 */
module.exports.updateCreateContact = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        // console.log(id);
        const datas = req.local.createContact;

        var images = [];
        if (req.files) {
            req.files.images.map(async (img) => {
                images.push(img.path)
            })
            datas.image = images;
        }

        const CreateContact_res = await CreateContactServices.updateCreateContact(id, datas);
        if (!CreateContact_res.status) return res.status(500).send(CreateContact_res);

        return res.status(200).send(CreateContact_res);
    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.updateCreateContact= ~ err:`, err.toString())
        return res.status(500).send({ status: false, error: err.toString() });
    }
};
/**
 * @description This is used for update status of createContact Object i.e. when created from web then status should be change 
 * @param {*} req.params.id from that get createContactId and from req.body.createStatus get status
 * @param {*} res 
 * @returns Object CreateContact_res { status : true,  CreateContacts: object }
 * - { status : false, error : string }
 */
module.exports.updateStatus = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const createStatus = req.body.createStatus;

        const status = "FALSE";

        const CreateContact_res = await CreateContactServices.updateCreateContact(id, { status: status, createStatus: createStatus });
        if (!CreateContact_res.status) return res.status(500).send(CreateContact_res);

        return res.status(200).send(CreateContact_res);
    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.updateStatus= ~ err:`, err.toString())
        return res.status(500).send({ status: false, error: err.toString() });
    }
};

module.exports.deleteContact = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const createStatus = req.body.createStatus;

        const status = "FALSE";

        const CreateContact_res = await CreateContactServices.updateCreateContact(id, { status: status, createStatus: createStatus });
        if (!CreateContact_res.status) return res.status(500).send(CreateContact_res);

        return res.status(200).send(CreateContact_res);
    }
    catch (err) {
        logger.error(`🚀 ~ module.exports.updateStatus= ~ err:`, err.toString())
        return res.status(500).send({ status: false, error: err.toString() });
    }
};


