const BranchServices = require('../Services/Branch.services');
const ClientServices = require('../Services/Client.services');
const CaseServices = require('../Services/Case.services');
const TransferBranchHistoryServices = require('../Services/TransferBranchHistory.services');
const OrganizationServices = require('../Services/Organization.services');
const { logger } = require('../logger');
const CreditOfficerServices = require('../Services/CreateServices')('CreditOfficer');

/**
 * @description This is used for transfer CreditOfficer from one branch to another branch 
 * - this is call from branch routes
 * @param {Object} req 
 * @param {*} res 
 * @returns Object userResult {status: true, users: Object}
 * -{status : false , err : string}
 */

module.exports.transferCreditOfficer = async (req, res) => {
    try {

        const datas = req.body; //creditOfficerId , newBranchId

        var deactivatedCreditOfficerArr = [];

        const organization = req.local.user.organization._id;

        const Organization = await OrganizationServices.getById(organization);
        if (!Organization.status) return res.status(500).send(Organization);

        const creditOfficer = await CreditOfficerServices.findById(datas.creditOfficerId);
        if (!creditOfficer.status || creditOfficer.error) return res.status(500).send(creditOfficer);

        const oldBranch = await BranchServices.getBranch(creditOfficer.data.branch._id);
        if (!oldBranch.status || oldBranch.error) return res.status(500).send(oldBranch);

        var oBranch = oldBranch.branches.toJSON();

        const newBranch = await BranchServices.getBranch(datas.newBranchId);
        if (!newBranch.status || newBranch.error) return res.status(500).send(newBranch);

        const checkInCaseCO = await CaseServices.getAllCases({ "creditOfficer._id": datas.creditOfficerId, "branch._id": oldBranch.branches._id });

        if (checkInCaseCO.cases.length != 0) {

            const checkInCase = await CaseServices.getAllCases({ caseStatus: { $in: ["CANCELLED", "DELIVERED", "PAYMENTPENDING", "CLOSED"] }, "creditOfficer._id": datas.creditOfficerId, "branch._id": oldBranch.branches._id });
            if (!checkInCase.status || checkInCase.error) return res.status(500).send(checkInCase);

            if (checkInCase.cases.length != 0) {
                if ("deactivatedCreditOfficer" in oBranch) {
                    deactivatedCreditOfficerArr = oBranch.deactivatedCreditOfficer;
                    deactivatedCreditOfficerArr.push(datas.creditOfficerId);

                    // console.log(deactivatedCreditOfficerArr, "In If");
                }
                else {
                    deactivatedCreditOfficerArr.push(datas.creditOfficerId);
                    // console.log(deactivatedCreditOfficerArr, "In Else");

                }
                const BranchResult = await BranchServices.editBranch({ _id: oldBranch.branches._id }, { deactivatedCreditOfficer: deactivatedCreditOfficerArr });
                if (!BranchResult.status || BranchResult.error) return res.status(500).send(BranchResult);

                const userResult = await CreditOfficerServices.updateById(datas.creditOfficerId, { branch: newBranch.branches })
                if (!userResult.status || userResult.error) return res.status(500).send(userResult);

                var caseListArr = [];
                await checkInCase.cases.map((caseId) => {
                    caseListArr.push(caseId._id);
                })

                var value = {
                    newBranch: newBranch.branches,
                    oldBranch: oldBranch.branches,
                    organization: Organization.Organizations,
                    creditOfficer: userResult.data,
                    dateTime: new Date(),
                    caseCount: checkInCase.cases.length,
                    caseList: caseListArr


                }
                const transferHistory = await TransferBranchHistoryServices.createTransferBranchHistory(value)
                if (!transferHistory.status || transferHistory.error) return res.status(500).send(transferHistory);

                // console.log("Match CaseStatus");
                return res.status(200).send(userResult);
            }
            else {
                // console.log("not match CaseStatus");
                return res.status(500).send({ status: false, error: "CreditOfficer can't transfer. Because case has been assigned to this CreditOfficer." });
            }
        }
        // console.log(oBranch);
        if ("deactivatedCreditOfficer" in oBranch) {
            deactivatedCreditOfficerArr = oBranch.deactivatedCreditOfficer;
            deactivatedCreditOfficerArr.push(datas.creditOfficerId);

            // console.log(deactivatedCreditOfficerArr, "In If");
        }
        else {
            deactivatedCreditOfficerArr.push(datas.creditOfficerId);
            // console.log(deactivatedCreditOfficerArr, "In Else");

        }
        const BranchResult = await BranchServices.editBranch({ _id: oldBranch.branches._id }, { deactivatedCreditOfficer: deactivatedCreditOfficerArr });
        if (!BranchResult.status || BranchResult.error) return res.status(500).send(BranchResult);

        const userResult = await CreditOfficerServices.updateById(datas.creditOfficerId, { branch: newBranch.branches })
        if (!userResult.status || userResult.error) return res.status(500).send(userResult);

        var value = {
            newBranch: newBranch.branches,
            oldBranch: oldBranch.branches,
            organization: Organization.Organizations,
            creditOfficer: userResult.data,
            dateTime: new Date(),
            caseCount: checkInCaseCO.cases.length,
            caseList: []
        }

        const transferHistory = await TransferBranchHistoryServices.createTransferBranchHistory(value)
        if (!transferHistory.status || transferHistory.error) return res.status(500).send(transferHistory);

        // console.log("not in Case")
        return res.status(200).send(userResult);

    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.getCreditOfficer = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        const id = req.params.id;

        const userRes = await CreditOfficerServices.findById(id);
        console.error(`🚀 ~ module.exports.getCreditOfficer= ~ userRes:`, userRes)
        if (!userRes.status) return res.status(500).send({ status: false, userRes });

        return res.status(200).send({ status: true, creditOfficer: userRes.data });

    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.createCreditOfficer = async (req, res) => {
    try {
        const datas = req.local.users;
        let orgId = datas.organization || req.local.user.organization;

        // For Remove Image
        if (datas.removeImg) {
            var rmImgArr = [];
            datas.removeImg.map(async (path, id) => {

                if (fs.existsSync(process.cwd() + "/" + path)) {
                    console.log("unlink img")
                    await fs.unlinkSync(process.cwd() + "/" + path);
                }
                else {
                    rmImgArr.push(path);
                }
            });
            if (rmImgArr.length != 0) {
                return res.send({ status: false, error: rmImgArr + " path doesn't exist For Remove Image" })
            }
        }
        delete datas.removeImg;

        if (datas.branch) {
            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status) return res.status(500).send(Branch);

            datas.branch = Branch.branches
        }

        const Organization = await OrganizationServices.getById(orgId);
        if (!Organization.status) return res.status(500).send(Organization);
        datas.organization = Organization.Organizations;

        const creditOfficerRes = await CreditOfficerServices.create(datas);
        if (!creditOfficerRes.status) return res.status(500).send(creditOfficerRes);

        const coResult = creditOfficerRes.data;
        const id = coResult._id.toString();
        if (req.file) {
            datas.profilePhoto = req.file.path;

            if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
            }
            const fileName = "profilePhoto_" + req.file.filename;

            let currentPath = process.cwd() + "/" + req.file.path;
            const destination = path.join(process.env.CDN_PATH, "users", id, fileName);

            // call function for move file
            await utilitiesController.moveFile(currentPath, destination);

            const filepath = path.join("users/", id, fileName);
            coResult.profilePhoto = filepath;
            coResult.save();
        }

        return res.status(200).send({ status: true, users: coResult });

    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.updateCreditOfficer = async (req, res) => {
    try {
        const id = req.params.id;
        const datas = req.local.users;
        const orgID = req.local.user.organization;

        const UserResult = await CreditOfficerServices.findById(id);
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        if (req.file) {
            datas.profilePhoto = req.file.path;

            if (!fs.existsSync(process.env.CDN_PATH + '/users/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/users/' + id, { recursive: true });
            }

            const fileName = "profilePhoto_" + req.file.filename;
            let currentPath = process.cwd() + "/" + req.file.path;
            const destination = path.join(process.env.CDN_PATH, "users", id, fileName);
            await utilitiesController.moveFile(currentPath, destination);

            const filepath = path.join("users", id, fileName);
            datas.profilePhoto = filepath;
        }

        if (!!datas.organization) {
            const Organization = await OrganizationServices.getById(datas.organization);
            if (!Organization.status) return res.status(500).send(Organization);
            datas.organization = Organization.Organizations.toJSON();
        }

        // For Remove Image
        if (datas.removeImg) {
            var rmImgArr = [];
            datas.removeImg.map(async (path, id) => {
                if (fs.existsSync(process.cwd() + "/" + path)) {
                    await fs.unlinkSync(process.cwd() + "/" + path);
                }
                else { rmImgArr.push(path); }
            });
        }
        delete datas.removeImg;

        if (datas.branch) {
            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status) return res.status(500).send(Branch);

            datas.branch = Branch.branches
        }

        const user_res = await CreditOfficerServices.updateById(id, datas);
        if (!user_res.status) return res.status(500).send(user_res);

        return res.status(200).send({ status: true, creditOfficer: user_res.data });
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for delete credit officer 
 * - This function call from user routes
 * @param {*} req 
 * @param {*} res 
 * @returns Object { status: true, message: "Credit Officer Deleted!" }
 * - { status: false, message: "Can't delete, because Case Assigned to this Credit Officer." }
 */
module.exports.deleteCreditOfficer = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        const id = req.params.id;
        const isdelete = true;

        // "organization._id": orgID,
        const caseResult = await CaseServices.getAllCaseLead({ "creditOfficer._id": id });
        if (!caseResult.status) return res.status(500).send({ status: false, caseResult });

        if (caseResult.cases.length == 0) {
            const userres = await CreditOfficerServices.updateById(id, { isDeleted: isdelete });
            if (!userres.status) return res.status(500).send({ status: false, userres });

            return res.status(200).send({ status: true, message: "Credit Officer Deleted!" });
        }
        else {
            return res.status(500).send({ status: false, message: "Can't delete, because Case Assigned to this Credit Officer." });
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

/**
 * @description This is used for get List Down all Credit Officers with count of its cases and case list 
 * - this function call from user routes
 * @param {*} req 
 * @param {*} res 
 * @returns Object UserResult { status: true, users: Array }
 * - { status : false, error : string }
 */
module.exports.getAllCreditOfficer = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id
        const query = {
            isDeleted: false,
            // "organization._id" : orgID
        };

        let count = 0;
        let creditOfficerArr = [];

        const { searchValue, client, branch } = req.query;

        if (!!searchValue) {
            query["$or"] = [
                { firstName: { $regex: searchValue, $options: 'i' } },
                { lastName: { $regex: searchValue, $options: 'i' } },
                { "branch.name": { $regex: searchValue, $options: 'i' } },
                { "branch.client.initial": { $regex: searchValue, $options: 'i' } },
                { "branch.client.name": { $regex: searchValue, $options: 'i' } }
            ]
        }
        if (client) query['branch.client._id'] = client;
        if (branch) query['branch._id'] = branch;

        const UserResultCountSearch = await CreditOfficerServices.count(query);
        logger.error(`🚀 ~ module.exports.getAllCreditOfficer= ~ query:`, UserResultCountSearch)
        if (!UserResultCountSearch.status || UserResultCountSearch.error) return res.status(500).send(UserResultCountSearch);
        count = UserResultCountSearch.count;

        const UserResult = await CreditOfficerServices.find({ query, limit: req.local.limit, start: req.local.startIndex });
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        await Promise.all(UserResult.data.map(async (user) => {
            // "organization._id": orgID,
            var caseResult = await CaseServices.getCount({ type: "CASE", "creditOfficer._id": user._id })

            // "organization._id": orgID,
            var leadResult = await CaseServices.getCount({ type: "LEAD", "creditOfficer._id": user._id })

            user = user.toJSON();
            user.caseCount = caseResult.cases;
            user.leadCount = leadResult.cases;
            user.totalCount = parseInt(caseResult.cases) + parseInt(leadResult.cases)

            creditOfficerArr.push(user);
        }))
        // console.log(count , " count ")
        return res.status(200).send({ status: true, users: creditOfficerArr, count: count });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get list of particular creditOfficer Transfer Branch History
 * - This is calling from user routes
 * @param {*} req 
 * @param {*} res 
 * @returns Object {status : true ,TransferBranchHistory : array}
 */
module.exports.getCreditOfficerBranchHistory = async (req, res) => {
    try {

        const orgID = req.local.user.organization._id;
        const creditOfficerId = req.params.id;

        // "organization._id": orgID,
        const TransferBranchHistory = await TransferBranchHistoryServices.getAll({ "creditOfficer._id": creditOfficerId });
        if (!TransferBranchHistory.status || TransferBranchHistory.error) return res.status(500).send(TransferBranchHistory);

        var TransferBranchHistoryArr = [];

        await Promise.all(TransferBranchHistory.TransferBranchHistory.map(async (data) => {
            data = data.toJSON();
            var caseArr = [];
            if (data.caseCount != "0") {
                await Promise.all(data.caseList.map(async caseId => {
                    // console.log(caseId, " ==== case Id ======")
                    const caseResult = await CaseServices.findCase(caseId);
                    // console.log(caseResult.cases, " ===== case Result =====")
                    await caseArr.push(caseResult.cases);
                }))
                // console.log(caseArr, "=====  case Array ======")
                data.caseListArr = caseArr;
            }
            caseArr = [];

            await TransferBranchHistoryArr.push(data);
        }))

        return res.status(200).send({ status: true, TransferBranchHistory: TransferBranchHistoryArr });
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};