const AssetServices = require('../Services/Asset.services');
const UserServices = require('../Services/User.services');
const CaseServices = require('../Services/Case.services');
const ReminderServices = require('../Services/Reminder.services');
const HolidayServices = require('../Services/Holiday.services');
const LeaveServices = require('../Services/Leave.services');
const AttendanceServices = require('../Services/Attendance.services');
const OrganizationServices = require('../Services/Organization.services');

const moment = require('moment');

//================================================ Asset ===================================
/**
 * @description This is used for get list of all Asset
 * @param {*} req 
 * @param {*} res 
 * @returns Object AssetResult { status: true, assets: Array }
 * - { status : false, error : string }                                                                                                                        
 */
module.exports.getAllAsset = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;

        const query = {};

        query['orgId'] = orgID;
        // query['status'] = true;
        // // query['isdelete'] = false;
        // // console.log(searchValue, " search Value ");
        const searchValue = req.query.searchValue;
        if (searchValue) {
            query["$or"] = [
                { name: { $regex: searchValue, $options: 'i' } }
            ]
        }

        const AssetResult = await AssetServices.getAllAssets(query, req.local.limit, req.local.startIndex);
        if (!AssetResult.status || AssetResult.error) return res.status(500).send(AssetResult);

        const AssetResultCountSearch = await AssetServices.getCount(query);
        if (!AssetResultCountSearch.status || AssetResultCountSearch.error) return res.status(500).send(AssetResultCountSearch);
        count = AssetResultCountSearch.assets;
        // return res.status(200).send(AssetResult);
        return res.status(200).send({ status: true, assets: AssetResult.assets, count: count });

    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Asset
 * @param {*} req.params.id from that get assetId 
 * @param {*} res 
 * @returns Object AssetResult { status: true, assets: object }
 * - { status : false, error : string }
 */
module.exports.getAsset = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id, " type: ", typeof id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const AssetResult = await AssetServices.getAsset(id);
        if (!AssetResult.status || AssetResult.error) return res.status(500).send(AssetResult);

        return res.status(200).send(AssetResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new Asset
 * @param {*} req.local.asset from that get asset Object
 * @param {*} res 
 * @returns Object createAssetResult { status: true, assets: object }
 * - { status : false, err : string }
 */
module.exports.createAsset = async (req, res) => {
    try {

        let datas = req.local.asset;
        datas.orgId = req.local.user.organization;

        if (req.file) {
            datas.assetImage = req.file.path;
        }

        const AssetResult = await AssetServices.getAllAssets({ assetNo: datas.assetNo });
        if (!AssetResult.status || AssetResult.error) return res.status(500).send(AssetResult);

        if (AssetResult.assets.length == 0) {

            const createAssetResult = await AssetServices.createAsset(datas)
            if (!createAssetResult.assets) return res.status(500).send({ status: false, createAssetResult });

            return res.status(200).send(createAssetResult);
        }
        else {
            return res.status(500).send({ status: false, error: 'Asset Already Created!' });
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update asset
 * @param {*} req.params.id from that get assetId 
 * @param {*} res 
 * @returns Object asset_res { status: true, assets: object }
 * - { status : false, err : string }
 */
module.exports.updateAsset = async (req, res) => {
    try {
        const id = req.params.id;
        let datas = req.local.asset;

        if (req.file) {
            datas.assetImage = req.file.path;
        }

        const asset_res = await AssetServices.updateAsset(id, datas);
        if (!asset_res.status) return res.status(500).send(asset_res);

        return res.status(200).send(asset_res);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for assign or return Asset to user
 * @param {*} req.params.id from that get assetId
 * @param {*} res 
 * @returns Object asset_res { status: true, assets: object }
 * - { status : false, err : string }
 */
module.exports.assetManagement = async (req, res) => {
    try {
        const id = req.params.id;

        let datas = req.body; // userId
        let obj = {};
        let arr = [];

        const AssetResult = await AssetServices.getAsset(id);
        if (!AssetResult.status || AssetResult.error) return res.status(500).send(AssetResult);

        const UserResult = await UserServices.findById(datas.userId);
        if (!UserResult.status || UserResult.error) return res.status(500).send(UserResult);

        const user = UserResult.users;
        // For Assign Asset
        if (datas.status == "ASSIGNED") {
            if (!(AssetResult.assets.users) || AssetResult.assets.users.length == 0) {

                obj['userId'] = user._id;
                obj['firstName'] = user.firstName;
                obj['lastName'] = user.lastName;
                obj['role'] = user.role;
                obj['status'] = "ASSIGNED";
                obj['issueDate'] = new Date();

                arr.push(obj);
                // console.log(arr);
                datas.users = arr;

                const asset_res = await AssetServices.updateAsset(id, datas);
                if (!asset_res.status) return res.status(500).send(asset_res);

                return res.status(200).send(asset_res);
            }

            arr = AssetResult.assets.users;

            obj['userId'] = user._id;
            obj['firstName'] = user.firstName;
            obj['lastName'] = user.lastName;
            obj['role'] = user.role;
            obj['status'] = "ASSIGNED";
            obj['issueDate'] = new Date();

            arr.push(obj);
            // console.log(arr);

            datas.users = arr;

            const asset_res = await AssetServices.updateAsset(id, datas);
            if (!asset_res.status) return res.status(500).send(asset_res);

            return res.status(200).send(asset_res);
        }
        // for Return Asset
        else {
            const returnResult = await Promise.all(AssetResult.assets.users.map(async (usr) => {
                if (usr.userId == datas.userId && usr.status == 'ASSIGNED') {

                    usr.status = "RETURN";
                    usr.returnDate = new Date();

                    await AssetResult.assets.save();
                    return usr;
                }
            }))
            // console.log(returnResult);
            return res.status(200).send(returnResult);
        }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
//================================================ employeewiseCases ===================================
/**
 * @description This is used for get case list employee wise
 * @param {*} req.params.id from that get user Id 
 * @param {*} res 
 * @returns Object { status: false, error: "Already have Registration Number" }
 * - return res.status(200).send({ status: true });
 */
module.exports.employeewiseCases = async (req, res) => {
    try {
        const id = req.params.id;

        let query = {};
        var count = 0;

        query['organization._id'] = req.local.user.organization._id;

        query['$or'] = [
            { "staff.pickupPerson._id": id },
            { "staff.maker._id": id },
            { "staff.checker._id": id },
            { "staff.valuers._id": id },
            { "staff.rateVerifier._id": id },
        ]

        const CaseResult = await CaseServices.getAllCases(query, req.local.startIndex, req.local.limit);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        const CaseResultCount = await CaseServices.getCount(query);
        if (!CaseResultCount.status || CaseResultCount.error) return res.status(500).send(CaseResultCount);
        count = CaseResultCount.cases;

        // console.log(CaseResult.cases, " Case Response")
        return res.status(200).send({ status: true, cases: CaseResult.cases, count: count });
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
//=========================================== Reminder =======================================
/**
 * @description This is used for get list of all Reminder
 * @param {*} req 
 * @param {*} res 
 * @returns Object ReminderResult { status: true, reminders: Array }
 * - { status : false, error : string }                                                                                                                        
 */
module.exports.getAllReminder = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        const createdBy = req.local.user._id;
        const query = {};

        query['orgId'] = orgID;
        query['createdBy'] = createdBy;
        // query['status'] = true;
        // // query['isdelete'] = false;
        // // console.log(searchValue, " search Value ");
        const searchValue = req.query.searchValue;
        if (searchValue) {
            query["$or"] = [
                { name: { $regex: searchValue, $options: 'i' } },
                { description: { $regex: searchValue, $options: 'i' } },
            ]
        }

        const ReminderResult = await ReminderServices.getAllReminder(query, req.local.limit, req.local.startIndex);
        if (!ReminderResult.status || ReminderResult.error) return res.status(500).send(ReminderResult);

        const AssetResultCountSearch = await ReminderServices.getCount(query);
        if (!AssetResultCountSearch.status || AssetResultCountSearch.error) return res.status(500).send(AssetResultCountSearch);
        count = AssetResultCountSearch.reminders;
        // return res.status(200).send(ReminderResult);
        return res.status(200).send({ status: true, reminders: ReminderResult.reminders, count: count });

    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Reminder
 * @param {*} req.params.id from that get assetId 
 * @param {*} res 
 * @returns Object ReminderResult { status: true, reminders: object }
 * - { status : false, error : string }
 */
module.exports.getReminder = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id, " type: ", typeof id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const ReminderResult = await ReminderServices.getReminder(id);
        if (!ReminderResult.status || ReminderResult.error) return res.status(500).send(ReminderResult);

        return res.status(200).send(ReminderResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new Reminder
 * @param {*} req.local.reminder from that get reminder Object
 * @param {*} res 
 * @returns Object createReminderResult { status: true, reminders: object }
 * - { status : false, err : string }
 */
module.exports.createReminder = async (req, res) => {
    try {

        let datas = req.local.reminder;
        datas.orgId = req.local.user.organization;

        // const ReminderResult = await ReminderServices.getAllReminder({ assetNo: datas.assetNo });
        // if (!ReminderResult.status || ReminderResult.error) return res.status(500).send(ReminderResult);

        // if (ReminderResult.reminders.length == 0) {

        datas.createdBy = req.local.user._id;

        const createReminderResult = await ReminderServices.createReminder(datas)
        if (!createReminderResult.reminders) return res.status(500).send({ status: false, createReminderResult });

        return res.status(200).send(createReminderResult);
        // }
        // else {
        //     return res.status(500).send({ status: false, error: 'Reminder Already Created!' });
        // }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update reminder
 * @param {*} req.params.id from that get reminderId 
 * @param {*} res 
 * @returns Object reminder_res { status: true, reminders: object }
 * - { status : false, err : string }
 */
module.exports.updateReminder = async (req, res) => {
    try {
        const id = req.params.id;
        const datas = req.local.reminder;

        const reminder_res = await ReminderServices.updateReminder(id, datas);
        if (!reminder_res.status) return res.status(500).send(reminder_res);

        return res.status(200).send(reminder_res);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
//=========================================== Holiday =======================================
/**
 * @description This is used for get list of all Holiday
 * @param {*} req 
 * @param {*} res 
 * @returns Object HolidayResult { status: true, holidays: Array }
 * - { status : false, error : string }                                                                                                                        
 */
module.exports.getAllHoliday = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        // const createdBy = req.local.user._id;
        const query = {};

        query['orgId'] = orgID;
        // query['createdBy'] = createdBy;

        const searchValue = req.query.searchValue;

        // console.log(searchValue, " search Value ");

        if (searchValue) {
            query["$or"] = [
                { title: { $regex: searchValue, $options: 'i' } },
                // { description: { $regex: searchValue, $options: 'i' } },
            ]
        }

        const HolidayResult = await HolidayServices.getAllHoliday(query, req.local.limit, req.local.startIndex);
        if (!HolidayResult.status || HolidayResult.error) return res.status(500).send(HolidayResult);

        const HolidayResultCount = await HolidayServices.getCount(query);
        if (!HolidayResultCount.status || HolidayResultCount.error) return res.status(500).send(HolidayResultCount);
        count = HolidayResultCount.holidays;
        // return res.status(200).send(HolidayResult);
        return res.status(200).send({ status: true, holidays: HolidayResult.holidays, count: count });

    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Holiday
 * @param {*} req.params.id from that get holidayID 
 * @param {*} res 
 * @returns Object HolidayResult { status: true, holidays: object }
 * - { status : false, error : string }
 */
module.exports.getHoliday = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id, " type: ", typeof id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const HolidayResult = await HolidayServices.getHoliday(id);
        if (!HolidayResult.status || HolidayResult.error) return res.status(500).send(HolidayResult);

        return res.status(200).send(HolidayResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new Holiday
 * @param {*} req.local.holiday from that get holiday Object
 * @param {*} res 
 * @returns Object createHolidayResult { status: true, holidays: object }
 * - { status : false, err : string }
 */
module.exports.createHoliday = async (req, res) => {
    try {

        let datas = req.local.holiday;

        // console.log(req.local.user.organization._id)

        datas.orgId = req.local.user.organization._id;

        // const HolidayResult = await HolidayServices.getAllHoliday({ assetNo: datas.assetNo });
        // if (!HolidayResult.status || HolidayResult.error) return res.status(500).send(HolidayResult);

        // if (HolidayResult.holidays.length == 0) {

        // datas.createdBy = req.local.user._id;

        const createHolidayResult = await HolidayServices.createHoliday(datas)
        if (!createHolidayResult.holidays) return res.status(500).send({ status: false, createHolidayResult });

        return res.status(200).send(createHolidayResult);
        // }
        // else {
        //     return res.status(500).send({ status: false, error: 'Holiday Already Created!' });
        // }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update holiday
 * @param {*} req.params.id from that get holidayId 
 * @param {*} res 
 * @returns Object holiday_res { status: true, holidays: object }
 * - { status : false, err : string }
 */
module.exports.updateHoliday = async (req, res) => {
    try {
        const id = req.params.id;
        const datas = req.local.holiday;

        const holiday_res = await HolidayServices.updateHoliday(id, datas);
        if (!holiday_res.status) return res.status(500).send(holiday_res);

        return res.status(200).send(holiday_res);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

//=========================================== Leave =======================================
/**
 * @description This is used for get list of all Leave
 * @param {*} req 
 * @param {*} res 
 * @returns Object LeaveResult { status: true, leaves: Array }
 * - { status : false, error : string }                                                                                                                        
 */
module.exports.getAllLeave = async (req, res) => {
    try {
        const orgID = req.local.user.organization._id;
        const approvedBy = req.local.user._id;
        const query = {};

        query['orgId'] = orgID;
        query['approvedBy'] = approvedBy;

        const searchValue = req.query.searchValue;

        // console.log(searchValue, " search Value ");

        if (searchValue) {
            query["$or"] = [
                { title: { $regex: searchValue, $options: 'i' } },
                // { description: { $regex: searchValue, $options: 'i' } },
            ]
        }

        const LeaveResult = await LeaveServices.getAllLeave(query, req.local.limit, req.local.startIndex);
        if (!LeaveResult.status || LeaveResult.error) return res.status(500).send(LeaveResult);

        const HolidayResultCount = await LeaveServices.getCount(query);
        if (!HolidayResultCount.status || HolidayResultCount.error) return res.status(500).send(HolidayResultCount);
        count = HolidayResultCount.leaves;
        // return res.status(200).send(LeaveResult);
        return res.status(200).send({ status: true, leaves: LeaveResult.leaves, count: count });

    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Leave
 * @param {*} req.params.id from that get leaveID 
 * @param {*} res 
 * @returns Object LeaveResult { status: true, leaves: object }
 * - { status : false, error : string }
 */
module.exports.getLeaveById = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id, " type: ", typeof id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const LeaveResult = await LeaveServices.getLeave(id);
        if (!LeaveResult.status || LeaveResult.error) return res.status(500).send(LeaveResult);

        return res.status(200).send(LeaveResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new Leave
 * @param {*} req.local.leave from that get leave Object
 * @param {*} res 
 * @returns Object createLeaveResult { status: true, leaves: object }
 * - { status : false, err : string }
 */
module.exports.createLeave = async (req, res) => {
    try {

        let datas = req.body;

        datas.approvedBy = req.local.user._id;

        datas.orgId = req.local.user.organization._id;

        // const LeaveResult = await LeaveServices.getAllHoliday({ assetNo: datas.assetNo });
        // if (!LeaveResult.status || LeaveResult.error) return res.status(500).send(LeaveResult);

        // if (LeaveResult.leaves.length == 0) {

        // datas.createdBy = req.local.user._id;

        const createLeaveResult = await LeaveServices.createLeave(datas)
        if (!createLeaveResult.leaves) return res.status(500).send({ status: false, createLeaveResult });

        return res.status(200).send(createLeaveResult);
        // }
        // else {
        //     return res.status(500).send({ status: false, error: 'Leave Already Created!' });
        // }
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update leave
 * @param {*} req.params.id from that get leaveId 
 * @param {*} res 
 * @returns Object leave_res { status: true, leaves: object }
 * - { status : false, err : string }
 */
module.exports.updateLeave = async (req, res) => {
    try {
        const id = req.params.id;
        const datas = req.body;

        const leave_res = await LeaveServices.updateLeave(id, datas);
        if (!leave_res.status) return res.status(500).send(leave_res);

        if (datas.leaveStatus == 'ACCEPTED') {

            const dateArr = await this.enumerateDaysBetweenDates(datas.fromDate, datas.toDate)
            // console.log("Dates : ", dateArr);

            await Promise.all(dateArr.map(async date => {
                datas.leaveId = id;
                datas.present = false;
                datas.date = date;
                const attendanceResult = await AttendanceServices.createAttendance(datas);
            }))
            // if (!attendanceResult.status) return res.status(500).send(attendanceResult);
        }

        return res.status(200).send(leave_res);
    }
    catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for get date between two requested Date 
 * @param {String} startDate 
 * @param {String} endDate 
 * @returns dates Array
 */
module.exports.enumerateDaysBetweenDates = async (startDate, endDate) => {

    startDate = moment(startDate);
    endDate = moment(endDate);

    var now = startDate, dates = [];

    while (now.isBefore(endDate) || now.isSame(endDate)) {
        dates.push(now.format('YYYY-MM-DD'));
        now.add(1, 'days');
    }
    return dates;
};

/**
 * @description This is used for get one Leave
 * @param {*} req.params.userId from that get userId 
 * @param {*} res 
 * @returns Object LeaveResult { status: true, leaves: object }
 * - { status : false, error : string }
 */
module.exports.getAllAttendance = async (req, res) => {
    try {
        const userId = req.params.userId;
        const fromDate = req.query.fromDate;
        const toDate = req.query.toDate;

        let query = {};

        query['userId'] = userId;
        query['date'] = {
            $gte: fromDate, //2021-10-01
            $lte: toDate //2021-10-31
        }
        // console.log(query)
        const AttendanceResult = await AttendanceServices.getAllAttendance(query, req.local.limit, req.local.startIndex);
        if (!AttendanceResult.status || AttendanceResult.error) return res.status(500).send(AttendanceResult);

        return res.status(200).send(AttendanceResult);
    }
    catch (e) {
        console.log("Error: ", e);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};


