const PackageServices = require('../Services/Package.services');
/**
 * @description This is used for get list of all packages 
 * @param {*} req 
 * @param {*} res 
 * @returns Object PakageResult {status : true , packages : Array }
 * -  {status : false , error : string }
 */
module.exports.getAllPackages = async (req, res) => {
    try {
        const PackageResult = await PackageServices.getAllPackages();

        if (!PackageResult.status || PackageResult.error) return res.status(500).send(PackageResult);

        return res.status(200).send(PackageResult);
    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get one Package
 * @param {*} req.params.id from that get packageId 
 * @param {*} res 
 * @returns Object PakageResult {status : true , packages : Object }
 * -  {status : false , error : string }
 */
module.exports.getPackage = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const PackageResult = await PackageServices.getPackage(id);
        if (!PackageResult.status || PackageResult.error) return res.status(500).send(PackageResult);

        return res.status(200).send(PackageResult);
    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new package  
 * @param {*} req.local.package  from that get package Object
 * @param {*} res 
 * @returns Object PakageResult {status : true , packages : Object }
 * -  {status : false , error : string }
 * -  { status: false, error: "Already Created!" }
 */
module.exports.createPackage = async (req, res) => {
    try {
        const datas = req.local.package;

        const alredyCreated = await PackageServices.getAllPackages({ title: datas.title })
        if (!alredyCreated.status || alredyCreated.error) return res.status(500).send(alredyCreated);
// console.log(alredyCreated)
        if (alredyCreated.packages.length == 0) {

            const PackageResult = await PackageServices.createPackage(datas);
            if (!PackageResult.status || PackageResult.error) return res.status(500).send(PackageResult);

            return res.status(200).send(PackageResult);
        }
        else {
            return res.status(500).send({ status: false, error: "Already Created!" });
        }
    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update Package
 * @param {*} req.params.id from that get packageId and from req.local.package get package Object
 * @param {*} res 
 * @returns Object Package_res {status : true , packages : Object }
 * -  {status : false , error : string }
 */
module.exports.updatePackage = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        // console.log(id);
        const datas = req.local.package;

        const Package_res = await PackageServices.updatePackage(id, datas);
        // console.log("Package: " + Package_res);
        if (!Package_res.status) return res.status(500).send(Package_res);

        return res.status(200).send(Package_res);
    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.deletePackage = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};


