const fs = require('fs');
const imagesToPdf = require('../Utils/imagesToPDF');
const resizeOptimizeImages = require('resize-optimize-images');
const path = require('path');
const {PROJECT_DIR } = require('../path');
const { logger} = require('../logger');

if (!fs.existsSync(process.env.CDN_PATH)) {
    fs.mkdirSync(process.env.CDN_PATH);
}

if (!fs.existsSync(process.env.CDN_PATH + '/imageToPdf')) {
    fs.mkdirSync(process.env.CDN_PATH + '/imageToPdf', { recursive: true });
}
/**
 * @description This is used For make PDF from uploaded Images and this is call from UtilitiesController
 * @param {Array} files
 * @returns Object { status: true, path: String, fileName:String }
 */
exports.makePDF = async (files, caseId = null) => {
    try {
        let imagespath = [];
        files.map((img, index) => {
            let path = img.path;
            imagespath.push(path);
        });

        if (!fs.existsSync(PROJECT_DIR + "/public/imageToPdf/")) {
            fs.mkdirSync(PROJECT_DIR + "/public/imageToPdf/", { recursive: true });
        }
        const filename = "/public/imageToPdf/" + Date.now() + "_" + files[0].size + ".pdf";
        const fullFileName = PROJECT_DIR + filename;

        const result = await imagesToPdf(imagespath, fullFileName)
        logger.error(`==> , ${JSON.stringify(imagespath)} ,  ${JSON.stringify(fullFileName)}, ${JSON.stringify(result)}`)
        if (result) {
            files.map((img, id) => {
                const res = fs.unlinkSync(PROJECT_DIR + "/" + img.path);
            });
        }

        let currentPath = fullFileName;
        if (caseId) {

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + caseId)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + caseId, { recursive: true });
            }
            const fileName = "document_" + Date.now() + "_" + files[0].size + ".pdf";
            const destination = path.join(process.env.CDN_PATH, "cases", caseId, fileName);
            // const destinationPath = process.cwd() + destination;

            fs.rename(currentPath, destination, function (err) {
                if (err) {
                    console.log("move File Error", err);
                    throw err
                } else {
                    console.log("Successfully moved the file!");
                }
            });

            currentPath = path.join("cases/", caseId, "/", fileName);
            return { status: true, path: currentPath, fileName: fileName };
        }
        // return { status: true, path: fullFileName, fileName: filename };
        return { status: true, path: filename, fileName: Date.now() + "_" + files[0].size + ".pdf" };
    }
    catch (e) {
        logger.error("images to pdf error: "+ e.toString());
        return { status: false, error: { type: 'PDF_MAKING', message: 'Error in Making PDF' } };
    }
}

/**
 * @description This is used For make PDF from uploaded Images and this is call from UtilitiesController
 * @param {Array} files
 * @returns Object { status: true, path: String, fileName:String }
 */
exports.generatePDF = async (files, pathToStore = path.join(process.cwd(), 'public', 'imageToPdf')) => {
    try {
        const imagesPath = files.map((img) => img.path);
        const filename = "" + Date.now() + "_" + files[0].size + ".pdf";
        const destination = path.join(pathToStore, filename);

        await fs.promises.mkdir(pathToStore, { recursive: true });
        await imagesToPdf(imagesPath, destination)

        files.forEach((img) => {
            fs.unlinkSync(path.join(process.cwd(), img.path));
        });
        return { status: true, path: destination, filename }
    }
    catch (e) {
        console.log("images to pdf error: ", e);
        return { status: false, error: { type: 'PDF_MAKING', message: 'Error in Making PDF.' } };
    }
}