const mongoose = require('mongoose');
const Asset = require('../Model/Asset');
/**
 * @description This is used for get list of All assets 
 * @param {*} query 
 * @returns Object { status: true, assets: Array }
 * - { status: false, error: String }
 */
module.exports.getAllAssets = async (query = {}) => {

    return Asset.find(query).then((assets) => {
        return { status: true, assets: assets };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One Asset By Id
 * @param {String} id 
 * @returns Object { status: true, assets: Object }
 * - { status: false, error: String }
 */
module.exports.getAsset = async (id) => {

    return Asset.findById(id).then((Asset) => {
        return { status: true, assets: Asset };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, assets: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", JSON.stringify(query))
    return Asset.countDocuments(query).then((Asset) => {
        return { status: true, assets: Asset };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
// module.exports.findAsset = async (id) => {

//     return Asset.findById(id).then((Asset) => {
//         return { status: true, assets: Asset };
//     }).catch((e) => {
//         return { status: false, error: e };
//     });

// };
/**
 * @description This is used for create Asset
 * @param {Object} assetData 
 * @returns Object { status: true, assets: Object }
 * - { status: false, error: String }
 */
module.exports.createAsset = async (assetData) => {

    return Asset.create(assetData)
        .then((result) => {
            // console.log("create Asset Service")
            return { status: true, assets: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Asset
 * @param {String} id 
 * @param {Object} assetData 
 * @returns Object { status: true, assets: Object }
 * - { status: false, error: String }
 */
module.exports.updateAsset = async (id, assetData) => {

    return Asset.findOneAndUpdate({ _id: id }, assetData,{new: true})
        .then((result) => {
            return { status: true, assets: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};

module.exports.aggregateAsset = async (assetData) => {
    // console.log(assetData.orgId)

    return Asset.aggregate([
        {
            "$match": {
                'orgId': mongoose.Types.ObjectId(assetData.orgId),
                "users": {
                    "$elemMatch": {
                        "$and": [
                            { "userId": assetData.userId },
                            { "status": assetData.status },
                        ]
                    }
                }
            }
        },
        {
            "$project": {
                "name": 1, "assetNo": 1,

                "users": {
                    "$filter": {
                        "input": "$users",
                        "as": "users",
                        "cond": {
                            "$and": [
                                { "$eq": ["$$users.userId", assetData.userId] },
                                { "$eq": ["$$users.status", assetData.status] }
                            ]
                        }
                    }
                }
            }
        }
    ])
        .then((result) => {
            return { status: true, assets: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
