const Client = require('../Model/Clients');
/**
 * @description This is used for create Client
 * @param {Object} clientData 
 * @returns Object { status: true, clients: Object }
 * - { status: false, error : string }
 */
module.exports.createClient = async (clientData) => {

    // console.log("In Services");
    // return {result : "result"};
    return Client.create(clientData)

        .then((result) => {
            // console.log("added Client")
            return { status: true, clients: result }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};
/**
 * @description This is used for get Client By Id 
 * @param {String} client_id 
 * @returns Object { status: true, clients: Object }
 * - { status: false, error : string }
 */
module.exports.getClientById = async (client_id) => {
    try {
        const client = await Client.findById(client_id);
        return { status: true, clients: client };
    }
    catch (err) {
        console.log(err);
        return { status: false, error: err };
    }
}
/**
 * @description This is used for get list of All Clients 
 * @param {*} query 
 * @returns Object { status: true, clients: Array }
 * - { status: false, error : string } 
 */
module.exports.getAllClients = async (query = {}, limit, startIndex) => {

    return Client.find(query).collation({ 'locale': 'en' }).sort({ name: 1 }).skip(startIndex).limit(limit)
        .then(result => {
            // console.log(query)
            return { status: true, clients: result };
        }).catch((e) => {
            console.log(e)
            return { status: false, error: e };
        });
}
module.exports.findAll = async (query = {}) => {

    return Client.find(query, { _id: 1, name: 1 })
        .then(result => {
            // console.log(query)
            return { status: true, clients: result };
        }).catch((e) => {
            console.log(e)
            return { status: false, error: e };
        });
}
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, clients: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Client.countDocuments(query).then((result) => {
        return { status: true, clients: result };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for update Client 
 * @param {String} id 
 * @param {Object} client_data 
 * @returns Object { status: true, clients: Array }
 * - { status: false, error : string } 
 */
module.exports.editClient = async (id, client_data) => {
    return Client.findOneAndUpdate({ _id: id }, client_data)
        .then((result) => {
            // console.log("Client Updated")
            return { status: true, clients: result }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        });
};

module.exports.deleteClient = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};



