const DocumentList = require('../Model/DocumentList');
/**
 * @description This is used for find all documents as per query from the collection 
 * @param {*} query 
 * @returns Object { status: true, documentLists: Array }
 * -  { status: false, error: string }
 */
module.exports.getAllDocumentList = async (query = {}, startIndex, limit) => {

    // console.log(query)
    return DocumentList.find(query).skip(startIndex).limit(limit)
        .then((DocumentList) => {
            return { status: true, documentLists: DocumentList };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, documentLists: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return DocumentList.countDocuments(query)
        .then((DocumentList) => {
            return { status: true, documentLists: DocumentList };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get One Document By Id 
 * @param {string} id 
 * @returns Object { status: true, documentLists: Object }
 * -  { status: false, error: string }
 */
module.exports.getDocumentList = async (id) => {

    return DocumentList.findById(id).then((DocumentList) => {
        return { status: true, documentLists: DocumentList };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get One Document
 * @param {*} query 
 * @returns Object { status: true, documentLists: Object }
 * -  { status: false, error: string }
 */
module.exports.getOneDocumentList = async (query) => {

    return DocumentList.findOne(query).then((DocumentList) => {
        return { status: true, documentLists: DocumentList };
    }).catch((e) => {
        return { status: false, error: e };
    });

};

/**
 * @description This is used for create new document to the collection
 * @param {Object} DocumentListData // data for create 
 * @returns Object { status: true, documentLists: Object }
 * -  { status: false, error: string }
 */
module.exports.createDocumentList = async (DocumentListData) => {

    return DocumentList.create(DocumentListData)
        .then((result) => {
            // console.log("create DocumentList Service")
            return { status: true, documentLists: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for edit document then update it from a requested id.
 * @param {string} id 
 * @param {Object} DocumentListData // data for update document 
 * @returns Object { status: true, documentLists: Object }
 * -  { status: false, error: string }
 */
module.exports.updateDocumentList = async (id, DocumentListData) => {

    return DocumentList.findOneAndUpdate({ _id: id }, DocumentListData, { new: true })
        .then((result) => {
            return { status: true, documentLists: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};