const Leave = require('../Model/Leave');
/**
 * @description This is used for get list of All leaves
 * @param {*} query 
 * @returns Object { status: true, leaves: Array }
 * - { status: false, error: string }
 */
module.exports.getAllLeave = async (query = {}, limit, startIndex) => {

    return Leave.find(query).skip(startIndex).limit(limit)
        .then((Leaves) => {
            return { status: true, leaves: Leaves };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, leaves: Object }
 *  { status: false, error: String }
 */
 module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Leave.countDocuments(query).then((Leaves) => {
        return { status: true, leaves: Leaves };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Leave By Id
 * @param {string} id 
 * @returns  Object { status: true, leaves: Object }
 * - { status: false, error: string }
 */
module.exports.getLeave = async (id) => {

    return Leave.findById(id).then((Leave) => {
        return { status: true, leaves: Leave };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one Leave
 * @param {*} query 
 * @returns Object { status: true, leaves: Object }
 * - { status: false, error: string }
 */
module.exports.findLeave = async (query = {}) => {

    // console.log(query);
    return Leave.findOne(query).then((Leave) => {
        return { status: true, leaves: Leave };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create Leave
 * @param {Object} LeaveData
 * @returns Object { status: true, leaves: Object }
 * - { status: false, error: string } 
 */
module.exports.createLeave = async (LeaveData) => {

    return Leave.create(LeaveData)
        .then((result) => {
            // console.log("create Leave Service")
            return { status: true, leaves: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Leave
 * @param {String} id 
 * @param {Object} LeaveData 
 * @returns Object { status: true, leaves: Object }
 * - { status: false, error: string } 
 */
module.exports.updateLeave = async (id, LeaveData) => {

    return Leave.findOneAndUpdate({ _id: id }, LeaveData,{new: true})
    
        .then((result) => {
            return { status: true, leaves: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};