const LocationHistory = require('../Model/LocationHistory');
/**
 * @description This is used for get list of all LocationHistory 
 * @param {*} query 
 * @returns Object  { status: true, locationHistory: Array }
 * - { status: false, error: string }
 */
module.exports.getAllLocationHistory = async (query = {}) => {
    return LocationHistory.find(query).then((LocationHistory) => {
        // console.log("query :", query);
        return { status: true, locationHistory: LocationHistory };
    }).catch((e) => {
        return { status: false, error: e };
    });
};
/**
 * @description This is used for get One LocationHistory By Id
 * @param {String} id 
 * @returns Object  { status: true, locationHistory: Object }
 * - { status: false, error: string }
 */
module.exports.getLocationHistory = async (id) => {

    return LocationHistory.findById(id).then((LocationHistory) => {
        return { status: true, locationHistory: LocationHistory };
    }).catch((e) => {
        return { status: false, error: e };
    });
};
/**
 * @description This is used for create LocationHistory 
 * @param {Object} LocationHistoryData 
 * @returns Object  { status: true, locationHistory: Object }
 * - { status: false, error: string }
 */
module.exports.createLocationHistory = async (LocationHistoryData) => {

    return LocationHistory.create(LocationHistoryData)
        .then((LocationHistory) => {
            // console.log("added Branch" + result)
            return { status: true, locationHistory: LocationHistory }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
/**
 * @description This is used for update LocationHistory 
 * @param {*} query 
 * @param {*} data 
 * @returns Object  { status: true, locationHistory: Object }
 * - { status: false, error: string }
 */
module.exports.editLocationHistory = async (query, data) => {
    const options = {
        // Return the document after updates are applied
        new: true,
        // Create a document if one isn't found. Required
        // for `setDefaultsOnInsert`
        upsert: true,
        setDefaultsOnInsert: true
    };
    return LocationHistory.findOneAndUpdate(query, { $push: { location: data } }, options)
        .then((LocationHistory) => {
            return { status: true, locationHistory: LocationHistory }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};

module.exports.deleteLocationHistory = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};



