const Organization = require('../Model/Organization');

module.exports.findOne = async (query) => {
    try {
        return Organization.findOne(query)
            .then((orgs) => {
                return { status: true, Organizations: orgs };
            }).catch((e) => {
                return { status: false, error: e };
            });
    }
    catch (e) {
        console.log("🚀 ~ file: Organization.services.js:18 ~ module.exports.findOne= ~ e:", e)
        return { status: false, error: e };
    }
};

/**
 * @description This is used for get list of All Organization or organization's Branch
 * @param {*} query 
 * @returns Object { status: true, Organizations: Array }
 * - { status: false, error: string }
 */
module.exports.findAll = async (query, { display = [], sort = { createdAt: -1 }, populate = [] } = { display: [], sort: { createdAt: -1 }, populate: [] }, startIndex, limit) => {

    return Organization.find(query, display).populate(populate).skip(startIndex).sort(sort).limit(limit)
        .then((Organizations) => {
            return { status: true, Organizations: Organizations };
        }).catch((e) => {
            console.log("🚀 ~ file: Organization.services.js:35 ~ .then ~ e:", e)
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get One Organization or organization's Branch By Id
 * @param {String} id 
 * @returns Object { status: true, Organizations: Object }
 * - { status: false, error: string }
 */
module.exports.getById = async (id) => {

    return Organization.findById(id).then((Organizations) => {
        return { status: true, Organizations: Organizations };
    }).catch((e) => {
        console.log("🚀 ~ file: Organization.services.js:50 ~ returnOrganization.findById ~ e:", e)
        return { status: false, error: e };
    });

}

/**
 * @description This is used for create Organization or organization's Branch
 * @param {Object} organizationData 
 * @returns Object { status: true, Organizations: Object }
 * - { status: false, error: string }
 */
module.exports.createOrganization = async (organizationData) => {

    return Organization.create(organizationData)
        .then((result) => {
            return { status: true, Organizations: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};

/**
 * @description This is used for update Organization or organization's Branch
 * @param {String} id 
 * @param {Object} organizationData 
 * @returns Object { status: true, Organizations: Object }
 * - { status: false, error: string }
 */
module.exports.updateOrganization = async (id, organizationData) => {
    try {
        const organization = await Organization.findOneAndUpdate({ _id: id }, organizationData, { new: true });
        // console.log("usr:"+user);
        if (!organization) return { status: false, error: "Organization not found" };

        return { status: true, Organizations: organization };

    }
    catch (err) {
        console.log(err);
        return { status: false, error: err };
    }
};


