const TransferBranchHistory = require('../Model/TransferBranchHistory');
/**
 * @description This is used for get All TransferBranchHistory
 * @param {*} query 
 * @returns Object { status: true, TransferBranchHistory: Array }
 *  { status: false, error: String }
 */
module.exports.getAll = async (query) => {
    // console.log(query)
    return TransferBranchHistory.find(query).then((TransferBranchHistory) => {
        return { status: true, TransferBranchHistory: TransferBranchHistory };
    }).catch((e) => {
        // console.log("In BranchServices", e);
        return { status: false, error: e };
    });
};
/**
 * @description This function is used for get one TransferBranchHistory from requested TransferBranchHistory id.  
 * @param {string} id 
 * @returns Object { status : true, TransferBranchHistory : object }
 * - { status: false, error: string }
 */
module.exports.getTransferBranchHistory = async (id) => {

    return TransferBranchHistory.findById(id).then((TransferBranchHistory) => {
        return { status: true, TransferBranchHistory: TransferBranchHistory };
    }).catch((e) => {
        // console.log("error", e);
        return { status: false, error: e };
    });
};
/**
 * @description This function is used for create TransferBranchHistory.
 * @param {Object} TransferBranchHistory 
 * @returns Object { status : true, TransferBranchHistory : object }
 * - { status: false, error: string }
 */
module.exports.createTransferBranchHistory = async (TransferBranchHistory_data) => {

    return TransferBranchHistory.create(TransferBranchHistory_data)
        .then((TransferBranchHistory) => {
            // console.log("added TransferBranchHistory" + result)
            return { status: true, TransferBranchHistory: TransferBranchHistory }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
/**
 * @description This function is used for update TransferBranchHistory data
 * @param {String} id 
 * @param {Object} branch_data 
 * @returns Object { status: true, TransferBranchHistory: Object}
 * - { status: false, err }
 */
module.exports.editTransferBranchHistory = async (id, TransferBranchHistory_data) => {
    return TransferBranchHistory.findOneAndUpdate({ _id: id }, TransferBranchHistory_data, { new: true })
        .then((TransferBranchHistory) => {
            return { status: true, TransferBranchHistory: TransferBranchHistory }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
