const jwt = require('jsonwebtoken');
const bcrypt = require('bcrypt');

const User = require('../Model/User');
const config = require('../Constants/Config');
/**
 * @description This is used for get list of All users
 * @param {*} sparam 
 * @returns Object { status: true, users: Array }
 * - { status: false, error: string }
 */
// module.exports.find = async (sparam) => {
//     try {
//         // console.log(sparam);
//         let query = { isdelete: false };
//         if (sparam.organization) {
//             query['organization._id'] = sparam.organization._id;
//         }
//         if (sparam.branch) {
//             query['branch._id'] = sparam.branch;
//         }
//         if (sparam.role) {

//             query['role'] = { $in: [sparam.role, 'ADMIN'] }
//             // query['role'] = sparam.role
//             // query['role'] = "ADMIN";
//         }
//         if (sparam.type) {
//             query['type'] = sparam.type;
//         }

//         return User.find(query).then((users) => {
//             // console.log("query :", query);
//             return { status: true, users: users };
//         }).catch((e) => {
//             return { status: false, error: e };
//         });
//     }
//     catch (e) {
//         console.log(e);
//         return { status: false, error: e };

//     }
// };
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, users: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return User.countDocuments({ isdelete: false, ...query }).then((Case) => {
        return { status: true, users: Case };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get List of All user
 * @param {Object} query 
 * @returns Object { status: true, users: Array }
 * - { status: false, error: string }
 */
module.exports.findAll = async (query = {}, limit, startIndex) => {

    try {
        return User.find({ isdelete: false, ...query }).collation({ 'locale': 'en' }).sort({ firstName: 1 }).skip(startIndex).limit(limit)
            .then((users) => {
                // console.log("query :", JSON.stringify(query));
                return { status: true, users: users };
            }).catch((e) => {
                console.log(e)
                return { status: false, error: e };
            });
    }
    catch (e) {
        console.log(e);
        return { status: false, error: e };
    }
};
/**
 * @description This is used for get One user By Id
 * @param {*} sparam 
 * @returns Object { status: true, users: Object }
 * - { status: false, error: string }
 */
module.exports.getOrganizationUser = async (sparam) => {
    try {
        // console.log(sparam)
        const user_id = sparam.organization.user_id;
        let query = {};
        if (sparam.organization._id) {
            // query.organization = {};
            query['organization._id'] = sparam.organization._id;
        }
        if (sparam.role) { query.role = sparam.role; }
        if (sparam.type) { query.type = sparam.type; }

        return User.findById(user_id).then((users) => {
            return { status: true, users: users };
        }).catch((e) => {
            return { status: false, error: e };
        });
    }
    catch (e) {
        console.log(e);
        return { status: false, error: e };
    }
};

// module.exports.findById = async (id) => {
// console.log("User Service" + ids)
//     return User.findById(id)
//         .then((user) => {
//             console.log(user,"user Service")
//             return { status: true, users: user }
//         })
//         .catch((e) => {
//             return { status: false, error: e };
//         })
// };
/**
 * @description This is used for get user List By Email 
 * @param {String} email 
 * @returns Object { status: true, users: Array }
 * - { status: false, error: string }
 */
module.exports.getUserByEmail = async (email) => {

    // console.log(email)
    return User.find({ "email": email.toLowerCase() }).select("+password")
        .then((user) => {
            return { status: true, users: user }
        })
        .catch((e) => {
            return { status: false, error: e };

        })
}
/**
 * @description This is used for get one User from Query
 * @param {Object} query 
 * @returns Object 
 */
module.exports.findOne = async (query) => {

    return User.findOne(query)
        .then((user) => {
            return { status: true, users: user }
        })
        .catch((e) => {
            console.log("UserService Error : ", e)
            return { status: false, error: e };

        })
}

module.exports.find = async (params = { query: {}, sort: { createdAt: -1 }, populate: [], start: 0, limit: 10, display: [], distinct: '_id' }) => {
    // { query: { }, sort = { createdAt: -1 }, populate = [], start = 0, limit = 10, display: [] }
    try {
        const {
            query = {},
            display = [],
            start = 0,
            limit = 10,
            populate = [],
            sort = '-createdAt',
            search = '',
            distinct = '_id'
        } = params;

        const count = await User.countDocuments(query);
        const data = await User.find(query, display)
            .populate(populate)
            .sort(sort)
            .skip(start)
            .limit(limit)
            // .distinct(distinct);

        const totalPages = Math.ceil(count / limit);

        return {
            status: true,
            data,
            count,
            totalPages,
        };
    }
    catch (e) {
        console.log("🚀 ~ file: User.services.js:192 ~ module.exports.find= ~ e:", e)
        return { status: false, error: e };
    }
};

/**
 * @description This is used for get One User By Id
 * @param {String} id 
 * @returns Object { status: true, users: Object }
 * - { status: false, error: string }
 */
module.exports.findById = async (id) => {
    return User.findById(id)
        .then((user) => {
            return { status: true, users: user }
        })
        .catch((e) => {
            console.log(e);
            return { status: false, error: e };
        })
};

module.exports.editMultipleUser = async (find_query, user_data) => {
    // console.log(find_query, " =======" , user_data);
    return User.updateMany(find_query, user_data, { new: true })
        .then((user) => {
            return { status: true, users: user }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};
/**
 * @description This is used for create User 
 * @param {Object} userData 
 * @returns Object { status: true, users: Object }
 * - { status: false, error: string }
 */
module.exports.createUser = async (userData) => {

    return User.create(userData)
        .then((user) => {
            // console.log("added Branch" + result)
            return { status: true, users: user }
        })
        .catch((err) => {
            // console.log(err , " INSERVICE")
            return { status: false, err }
        })
};
/**
 * @description This is used for update User 
 * @param {String} id 
 * @param {Object} user_data 
 * @returns Object { status: true, users: Object }
 * - { status: false, error: string }
 */
module.exports.editUser = async (id, user_data) => {

    return User.findOneAndUpdate({ _id: id }, user_data, { new: true })
        .then((user) => {
            // console.log("update Branch",branch)
            return { status: true, users: user }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
}
module.exports.updateUser = async (findQuery, user_data) => {

    return User.findOneAndUpdate(findQuery, user_data, { new: true })
        .then((user) => {
            // console.log("update Branch",branch)
            return { status: true, users: user }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
}

/**
 * @description This is used for Generate JWT Token
 * @param {*} user 
 * @returns data, signature, { expiresIn: expiration }
 */
module.exports.generateJWTToken = async (user) => {
    const data = {
        _id: user._id,
        email: user.email,
    };
    const signature = config.secret;
    // const expiration = '120s';
    const expiration = '7d';

    return jwt.sign(data, signature, { expiresIn: expiration });
};

/**
 * @description This is used Generate JWT Token For reset password
 * @param {String} email 
 * @returns jwt token
 */
module.exports.generateJWTTokenForResetPassword = async (email, expiration = "5m") => {
    const data = { email: email };
    const signature = config.secret;

    return jwt.sign(data, signature, { expiresIn: expiration });
};

/**
 * @description This is used for Hashing Password
 * @param {String} password 
 * @returns Object { status: true, hashPassword }
 */
module.exports.hashPassword = async (password) => {
    try {
        const salt = await bcrypt.genSalt(10);
        const hashPassword = await bcrypt.hash(password, salt);
        return { status: true, hashPassword };
    }
    catch (e) {
        return { status: false, error: e }
    }
}

module.exports.matchPassword = async (userId, password) => {
    try {

        const user = await User.findById(userId).select('+password').then((user) => { return user; }).catch((e) => { return null; })
        if (!user) { return false; }

        const isMatch = await bcrypt.compare(password, user.password);
        if (!isMatch) { return false; }

        return true;
    }
    catch (e) {
        logger.error(`🚀 ~ module.exports.matchPassword= ~ e:`+ e.toString())
        return false
    }
}