const fs = require('fs');
const path = require('path');

const CDN_PATH = process.env.CDN_PATH;
/**
 * @description This is used for create file from base64 string 
 * @param {Array} base64  base64 string
 * @param {*} fileName 
 * @param {*} targetDirectory  don`t include cdn path in it
 * @returns Object { status: true, result: array }
 * - { status: false, error: "file not found" }
 */
async function base64toFile(base64, fileName, filePath) {
    try {
        const filepath = path.join(filePath, fileName);
        const targetDirectory = path.join(CDN_PATH, filePath);
        fs.mkdirSync(targetDirectory, { recursive: true });

        const buffer = Buffer.from(base64.split('base64,')[1], 'base64')
        const destination = path.join(targetDirectory, fileName);
        await fs.writeFileSync(destination, buffer);

        return { status: true, filepath }
    }
    catch (e) {
        return { status: false, error: e }
    }
}


module.exports = base64toFile
