const { MongoClient, ObjectId, ObjectID } = require('mongodb');
const fs = require('fs');
const path = require('path');
const CDN = '../CDN/';
var args = process.argv.slice(2);

async function main() {

    const client = new MongoClient("mongodb://192.168.1.36:27017");
    try {
        // Connect to the MongoDB cluster
        await client.connect();
        const cases = client.db('realvalue').collection("cases");

        // Find the existing case document
        const caseNumber = args[0];
        const existingCase = await cases.findOne({ caseNumber });

        if (!existingCase) {
            console.log(`No case found with caseNumber: ${caseNumber}`);
            return;
        }

        const oldId = existingCase._id;
        const newId = new ObjectId()
        // Duplicate the existing document with a new _id
        const duplicatedCase = { ...existingCase, _id: newId };

        const updatedJsonObject = updateIdInJsonString(JSON.stringify(duplicatedCase), oldId, newId);

        // Update caseNumber and createdAt in the duplicated document
        updatedJsonObject._id = newId;
        updatedJsonObject.caseNumber = await generateNewCaseNumber(cases);
        updatedJsonObject.createdAt = new Date();
        updatedJsonObject.createdBy = ObjectID(updatedJsonObject.createdBy);

        updatedJsonObject.outward = { status: false };
        updatedJsonObject.caseStatus = "CASE";
        updatedJsonObject.invoiceId = null;

        // Insert the duplicated document into the collection
        await cases.insertOne(updatedJsonObject);

        console.log(`Case ${duplicatedCase.caseNumber} duplicated successfully with new caseNumber: ${updatedJsonObject.caseNumber}`);

        // Copy the entire folder
        await copyFolderRecursive(CDN + 'cases/' + oldId, CDN + 'cases/' + newId);

    } catch (e) {
        console.error(e);
    } finally {
        await client.close();
    }
}


// Function to generate a new caseNumber
async function generateNewCaseNumber(coll) {
    // Get the last created case from the collection
    const lastCase = await coll.findOne({}, { sort: { createdAt: -1 } });

    var caseNo = lastCase.caseNumber;
    var caseNumber = parseInt(caseNo) + 1;
    const str = "" + caseNumber, pad = "000000"
    var answer = pad.substring(0, pad.length - str.length) + str;
    return answer;
}

function updateIdInJsonString(jsonString, oldId, newId) {
    try {
        const updatedJsonString = jsonString.replace(new RegExp(`\\b${oldId}\\b`, 'gi'), newId);
        const jsonObject = JSON.parse(updatedJsonString);
        return jsonObject;
    } catch (error) {
        console.error('Error parsing or updating JSON:', error);
        return null;
    }
}

function copyFolderRecursive(sourcePath, destinationPath) {
    // Ensure the destination directory exists
    if (!fs.existsSync(destinationPath)) {
        fs.mkdirSync(destinationPath);
    }

    // Read the contents of the source directory
    const files = fs.readdirSync(sourcePath);
    console.log("->OLD: ", sourcePath, "\n New: ", destinationPath);

    files.forEach((file) => {
        const sourceFilePath = path.join(sourcePath, file);
        const destinationFilePath = path.join(destinationPath, file);
        // Check if the current item is a directory
        const isDirectory = fs.statSync(sourceFilePath).isDirectory();

        if (isDirectory) {
            // Recursively copy subdirectories
            copyFolderRecursive(sourceFilePath, destinationFilePath);
        } else {
            // Copy individual files
            fs.copyFileSync(sourceFilePath, destinationFilePath);
        }
    });
}

main().catch(console.error);