const express = require('express');
const router = express.Router();
const DynamicDropDownController = require('../Controllers/DynamicDropDown.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const pagination = require('../Middleware/pagination');
const validation = require('../Middleware/validation');
const isPermission = require('../Middleware/isPermission');

// For get list of dropdowns 
router.get('/dropdowns', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "list" }) }, pagination, DynamicDropDownController.getAllDynamicDropDowns);

// For get only One dropdown 
router.get('/dropdowns/:name', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "view" }) }, DynamicDropDownController.getDynamicDropDown);

// For create dropdown 
router.post('/dropdowns', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "create" }) }, validation.dropDownValidation, DynamicDropDownController.createDynamicDropDown);

// For update whole options of dropdown 
router.put('/dropdowns/:name', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "edit" }) }, DynamicDropDownController.updateDynamicDropDown);

// For add value in Exist option of dropdown 
router.post('/dropdowns/:name', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "edit" }) }, DynamicDropDownController.editDynamicDropDown);

// router.delete('/dropdowns/:id', isAuth, isAdmin,(req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "list" }) }, DynamicDropDownController.deleteDynamicDropDown);

// router.put('/dropdownsupdateMany',isAuth, DynamicDropDownController.updateAllDynamicDropDown);

module.exports = router;

