const express = require('express');
const router = express.Router();
const OrganizationController = require('../Controllers/Organization.controller');
const isAuth = require('../Middleware/isAuth');
const isSuperAdmin = require('../Middleware/isSuperAdmin');
const validation = require('../Middleware/validation');
const uploadFile = require('../Middleware/uploadFile');
const isPermission = require('../Middleware/isPermission');
const isBranchAdmin = require('../Middleware/isBranchAdmin');

// -----------------    ORGANIZATION    -------------------------
// For get list of All organization 
router.get('/Organizations', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "list" }) }, OrganizationController.getAllOrganizations);
// For get one organization
router.get('/Organizations/:id', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "view" }) }, OrganizationController.getOrganization);
// For create new Organization 
router.post('/Organizations', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "create" }) }, uploadFile.createOrganization, validation.createOrganizationValidation, OrganizationController.createOrganization);
// For update Organization 
router.put('/Organizations/:id', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "edit" }) }, uploadFile.createOrganization, validation.createOrganizationValidation, OrganizationController.updateOrganization);
// For enable Organization
router.post('/Organizations/:id/status', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "edit" }) }, OrganizationController.enableOrganizationStatus);
// For disable Organization
router.delete('/Organizations/:id/status', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "organization", value: "edit" }) }, OrganizationController.disableOrganizationStatus);


// -----------------    BRANCH  -------------------------
// For get List of All branches of an Organization 
router.get('/Organizations/:id/Branch', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "dropDown", value: "list" }) }, OrganizationController.getBranchesOrganizationWise);
// For create organization's branch 
router.post('/Organizations/:orgId/Branch', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "orgbranch", value: "create" }) }, uploadFile.createOrganization, validation.createOrgBranchValidation, OrganizationController.createOrganizationBranch);
// For update organization's branch
router.put('/Organizations/:orgId/Branch/:branchId', isAuth, isBranchAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "orgbranch", value: "edit" }) }, uploadFile.createOrganization, validation.createOrgBranchValidation, OrganizationController.updateOrganizationBranch);
// For enable Organization
router.post('/Organizations/:id/status', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "orgbranch", value: "edit" }) }, OrganizationController.enableOrganizationStatus);
// For disable Organization
router.delete('/Organizations/:id/status', isAuth, isSuperAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "orgbranch", value: "edit" }) }, OrganizationController.disableOrganizationStatus);



// ----------------- COMMON -------------------------
// For create owner in organization/branch
router.post('/Organizations/:id/owner', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "view" }) }, OrganizationController.updateOwnerOrgBranch);
router.delete('/Organizations/:id/owner/:ownerId', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "view" }) }, OrganizationController.deleteOwnerOrgBranch);




module.exports = router;

