const express = require('express');
const router = express.Router();

const ReportController = require('../Controllers/Report.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const pagination = require('../Middleware/pagination');
const validation = require('../Middleware/validation');

// For get list of all Roles 
router.get('/reports/inactiveClients', isAuth, ReportController.inactiveCaseList);
router.get('/reports/userWiseTaskCount', isAuth, ReportController.userWiseTaskCount);
router.get('/reports/userWiseTask', isAuth, ReportController.userWiseTask);
router.get('/reports/topUsers', isAuth, ReportController.topUsers);

// web dashboard
router.get('/reports/dashboardCounts', isAuth, ReportController.dashboardCounts);
router.get('/reports/dashboard/userWiseMyTask', isAuth, ReportController.usersMyTaskCount);
router.get('/reports/dashboard/workTypeOpenTask', isAuth, ReportController.workTypeOpenTaskCount);
router.get('/reports/dashboard/statusWiseReportCount', isAuth, ReportController.statusWiseReportCount);
router.get('/reports/dashboard/clientWiseReportCount', isAuth, ReportController.clientWiseReportCount);
router.get('/reports/dashboard/tasksByWorkTypeCount', isAuth, ReportController.getTaskCountByWorkType);
router.get('/reports/dashboard/staffWiseReportCount', isAuth, ReportController.staffWiseReportCount);
router.get('/reports/dashboard/clientWiseInvoice', isAuth, ReportController.clientWiseInvoice);
router.get('/reports/dashboard/staffWiseInvoice', isAuth, ReportController.staffWiseInvoice);

router.get('/reports/payment-dashboard', isAuth, ReportController.paymentDashboard);

router.get('/reports/client-visit-count', isAuth, ReportController.clientVisitCounts);
router.get('/reports/compareWithTask', isAuth, ReportController.compareWithTask);

// users report
router.get('/reports/statusWiseLeadCount', isAuth, ReportController.statusWiseLeadCount)
router.get('/reports/statusWiseCaseCount', isAuth, ReportController.statusWiseCaseCount);
router.get('/reports/clientWiseCaseCount', isAuth, ReportController.clientWiseCaseCount);

// master reports
router.get('/reports/customReport', pagination, ReportController.customReport);

// saved reports api
router.get('/saved-reports', isAuth, pagination, ReportController.getAllSavedReports);
router.get('/saved-reports/:id', isAuth, ReportController.getSavedReport);
router.post('/saved-reports', isAuth, validation.SavedReport, ReportController.createSaveReport);
router.put('/saved-reports/:id', isAuth, validation.SavedReport, ReportController.updateSaveReport);
router.delete('/saved-reports/:id', isAuth, ReportController.deleteSavedReport);

// sales person Dashboard routes
router.get('/reports/completed-cases-by-clients', isAuth, ReportController.getCompletedCasesByClient);
router.get('/reports/completed-cases-by-branches', isAuth, ReportController.getCompletedCasesByBranch)
router.get('/reports/lead-by-clients', isAuth, ReportController.getLeadByClient);
router.get('/reports/lead-by-branches', isAuth, ReportController.getLeadByBranch);
router.get('/reports/invoice-by-clients', isAuth, ReportController.getInvoiceByClient);
router.get('/reports/invoice-by-branches', isAuth, ReportController.getInvoiceByBranch);
router.get('/reports/cancelled-cases-by-clients', isAuth, ReportController.getCancelledCasesByClient);
router.get('/reports/cancelled-cases-by-branches', isAuth, ReportController.getCancelledCasesByBranch);
router.get('/reports/outstanding-amount-by-clients', isAuth, ReportController.getOutstandingAmountByClient);
router.get('/reports/outstanding-amount-by-branches', isAuth, ReportController.getOutstandingAmountByBranch);
router.get('/reports/outstanding-barchart-list', isAuth, ReportController.getPendingAmountBarGraph);
// router.get('/reports/inactive-clients', isAuth, ReportController.inactiveCaseListByBranch);


module.exports = router;