const express = require('express');
const router = express.Router();
const ZoneController = require('../Controllers/Zone.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const pagination = require('../Middleware/pagination');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');

// For get list of all Zones
router.get('/zones', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "zone", value: "list" }) }, pagination, ZoneController.getAllZones);

// For get only One Zone
router.get('/zones/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "zone", value: "view" }) }, ZoneController.getZone);

//For create new Zone
router.post('/zones', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "zone", value: "create" }) }, validation.zoneValidation, ZoneController.createZone);

// For update Zone
router.put('/zones/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "zone", value: "edit" }) }, validation.zoneValidation, ZoneController.updateZone);

// For get list of branches for particular zone
router.get('/zones/:id/branches', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "branch", value: "list" }) }, ZoneController.getAllBranchesByZone);

// For Fees calculation
router.post('/calculation', isAuth, ZoneController.calculation);

// For delete zone
router.delete('/deleteZone/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "zone", value: "delete" }) }, ZoneController.deleteZone);

// ========================= Empanelment Routes ================================
// For check Empanelment No Exist or not 
router.get('/Empanelments', isAuth,  (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "list" }) }, pagination, ZoneController.getEmpanelments);

// For check Empanelment No Exist or not 
router.post('/checkEmpanelmentNoExist', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "view" }) }, ZoneController.checkEmpanelmentNoExist);

// For get Empanelment details 
router.get('/getEmpanelment/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "view" }) }, ZoneController.getEmpanelment);

// For update Empanelment details
router.put('/updateEmpanelment/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "edit" }) }, ZoneController.updateEmpanelment);

// For delete Empanelment Letter
router.put('/deleteEmpanelment/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "delete" }) }, ZoneController.deleteEmpanelment);

// For share Empanelment Letter 
router.post('/shareEmpanelmentLetter/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "empanelment", value: "edit" }) }, ZoneController.shareEmpanelment);

module.exports = router;