const { MongoClient } = require('mongodb');

// MongoDB connection URI
const uri = 'mongodb://192.168.1.36:27017';  // Update with your MongoDB URI

// Database and collection names
const dbName = 'realvalue';
const collectionName = 'cases';

// Create a new MongoClient
const client = new MongoClient(uri, { useNewUrlParser: true, useUnifiedTopology: true });

async function updateCases() {
    try {
        // Connect to the MongoDB client
        await client.connect();
        console.log('Connected to MongoDB');

        // Get the database and collection
        const db = client.db(dbName);
        const collection = db.collection(collectionName);

        // Fetch all documents from the 'cases' collection
        const cases = await collection.find({ 'outward.date': { $exists: true }, 'outward.datetime': { $exists: false } }).limit(1000).toArray();

        const currentDate = new Date();

        for (const caseDoc of cases) {
            const outwardDateString = caseDoc.outward?.date;  // Get outward date string

            if (outwardDateString) {
                let formattedDate;

                // Check if the date is in "DD/MM/YYYY" format
                if (/^\d{2}\/\d{2}\/\d{4}$/.test(outwardDateString)) {
                    // Convert "DD/MM/YYYY" format to a JavaScript Date object
                    const [day, month, year] = outwardDateString.split('/');
                    formattedDate = new Date(`${year}-${month}-${day}T00:00:00Z`);  // Creating ISO string format
                } else {
                    // If it's not in "DD/MM/YYYY" format, assume it's in the standard JavaScript date string format
                    formattedDate = new Date(outwardDateString);  // Convert to a JavaScript Date object
                }

                // Check if the date is valid
                if (!isNaN(formattedDate)) {
                    // Update the document to add the datetime field
                    await collection.updateOne(
                        { _id: caseDoc._id },
                        { $set: { "outward.datetime": formattedDate } }  // Add the datetime field inside outward object
                    );
                    console.log(`ID: ${caseDoc.caseNumber}, old: ${outwardDateString}, new datetime: ${formattedDate}`);
                } else {
                    console.log(`Invalid date format for Case ID: ${caseDoc._id}`);
                }
            } else {
                console.log(`No outward.date field for Case ID: ${caseDoc._id}`);
            }
        }
    } catch (err) {
        console.error('Error connecting to MongoDB or updating documents', err);
    } finally {
        // Close the MongoDB connection
        await client.close();
        console.log('MongoDB connection closed');
    }
}

// Run the update function
updateCases();
